@@ -35,6 +35,7 @@
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
+#include "camel-mapi-settings.h"
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-summary.h"
 #include "camel-mapi-notifications.h"
@@ -52,11 +53,8 @@
 #define d(x) printf("%s:%s:%s \n", G_STRLOC, G_STRFUNC, x)
 
 struct _CamelMapiStorePrivate {
-	gchar *profile;
 	ExchangeMapiConnection *conn;
 
-	gchar *base_url;
-
 	GHashTable *id_hash; /*get names from ids*/
 	GHashTable *name_hash;/*get ids from names*/
 	GHashTable *container_hash;
@@ -67,13 +65,23 @@ struct _CamelMapiStorePrivate {
 	gpointer notification_data; /* pointer to a notification data; can be only one */
 };
 
-G_DEFINE_TYPE (CamelMapiStore, camel_mapi_store, CAMEL_TYPE_OFFLINE_STORE)
+/* Forward Declarations */
+static void camel_subscribable_init (CamelSubscribableInterface *interface);
+
+G_DEFINE_TYPE_WITH_CODE (
+	CamelMapiStore,
+	camel_mapi_store,
+	CAMEL_TYPE_OFFLINE_STORE,
+	G_IMPLEMENT_INTERFACE (
+		CAMEL_TYPE_SUBSCRIBABLE,
+		camel_subscribable_init))
 
 /* service methods */
 static void mapi_store_constructed (GObject *object);
 static gchar	*mapi_get_name(CamelService *, gboolean );
 static gboolean	mapi_connect_sync(CamelService *, GCancellable *cancellable, GError **);
 static gboolean	mapi_disconnect_sync(CamelService *, gboolean , GCancellable *cancellable, GError **);
+static CamelAuthenticationResult mapi_authenticate_sync (CamelService *, const gchar *mechanism, GCancellable *, GError **);
 static GList	*mapi_query_auth_types_sync(CamelService *, GCancellable *cancellable, GError **);
 
 /* store methods */
@@ -363,7 +371,7 @@ remove_path_from_store_summary (const gchar *path, gpointer value, CamelMapiStor
 				fi->display_name = g_strdup (fi->display_name + 1);
 		}
 
-		camel_store_folder_unsubscribed (CAMEL_STORE (mstore), fi);
+		camel_subscribable_folder_unsubscribed (CAMEL_SUBSCRIBABLE (mstore), fi);
 		camel_store_folder_deleted (CAMEL_STORE (mstore), fi);
 		camel_folder_info_free (fi);
 
@@ -510,7 +518,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 
 				if (!subscription_list) {
 					camel_store_folder_created (CAMEL_STORE (store), info);
-					camel_store_folder_subscribed (CAMEL_STORE (store), info);
+					camel_subscribable_folder_subscribed (CAMEL_SUBSCRIBABLE (store), info);
 				}
 			}
 
@@ -674,8 +682,11 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 		}
 
 		if (!subscription_list && !(si->flags & CAMEL_MAPI_FOLDER_MAIL) && si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED &&
-												si->flags & CAMEL_MAPI_FOLDER_PUBLIC)
+												si->flags & CAMEL_MAPI_FOLDER_PUBLIC) {
+			camel_store_summary_info_free ((CamelStoreSummary *) mapi_store->summary, si);
 			continue;
+		}
+
 		if (!strcmp(name, camel_mapi_store_info_full_name (mapi_store->summary, si))
 		     || match_path (path, camel_mapi_store_info_full_name (mapi_store->summary, si))) {
 
@@ -697,6 +708,24 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 			fi->total = si->total;
 			fi->flags = si->flags;
 
+			if ((fi->flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_NORMAL) {
+				guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, fi->full_name);
+				switch (folder_type) {
+				case MAPI_FOLDER_TYPE_CONTACT:
+					fi->flags |= CAMEL_FOLDER_TYPE_CONTACTS;
+					break;
+				case MAPI_FOLDER_TYPE_APPOINTMENT:
+					fi->flags |= CAMEL_FOLDER_TYPE_EVENTS;
+					break;
+				case MAPI_FOLDER_TYPE_MEMO:
+					fi->flags |= CAMEL_FOLDER_TYPE_MEMOS;
+					break;
+				case MAPI_FOLDER_TYPE_TASK:
+					fi->flags |= CAMEL_FOLDER_TYPE_TASKS;
+					break;
+				}
+			}
+
 			g_ptr_array_add (folders, fi);
 		}
 		camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
@@ -724,18 +753,17 @@ static gboolean
 mapi_forget_folder (CamelMapiStore *mapi_store, const gchar *folder_name, GError **error)
 {
 	CamelService *service;
-	const gchar *user_data_dir;
+	const gchar *user_cache_dir;
 	gchar *state_file;
 	gchar *folder_dir, *storage_path;
 	CamelFolderInfo *fi;
 
 	service = CAMEL_SERVICE (mapi_store);
-	user_data_dir = camel_service_get_user_data_dir (service);
+	user_cache_dir = camel_service_get_user_cache_dir (service);
 
-	storage_path = g_strdup_printf ("%s/folders", user_data_dir);
+	storage_path = g_build_filename (user_cache_dir, "folders", NULL);
 
-	/* Fixme Path - e_*-to_path */
-	folder_dir = g_strconcat (storage_path, "/", folder_name, NULL);
+	folder_dir = g_build_filename (storage_path, folder_name, NULL);
 	g_free (storage_path);
 
 	if (g_access(folder_dir, F_OK) != 0) {
@@ -743,7 +771,7 @@ mapi_forget_folder (CamelMapiStore *mapi_store, const gchar *folder_name, GError
 		return TRUE;
 	}
 
-	state_file = g_strdup_printf ("%s/cmeta", folder_dir);
+	state_file = g_build_filename (folder_dir, "cmeta", NULL);
 	g_unlink (state_file);
 	g_free (state_file);
 
@@ -832,9 +860,6 @@ mapi_store_finalize (GObject *object)
 
 	priv = CAMEL_MAPI_STORE (object)->priv;
 
-	g_free (priv->profile);
-	g_free (priv->base_url);
-
 	if (priv->id_hash != NULL)
 		g_hash_table_destroy (priv->id_hash);
 
@@ -875,33 +900,19 @@ mapi_store_can_refresh_folder (CamelStore *store,
                                GError **error)
 {
 	CamelService *service;
-	CamelURL *url;
+	CamelSettings *settings;
+	gboolean check_all;
 
 	/* skip unselectable folders from automatic refresh */
 	if (info && (info->flags & CAMEL_FOLDER_NOSELECT) != 0)
 		return FALSE;
 
 	service = CAMEL_SERVICE (store);
-	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
 
-	return CAMEL_STORE_CLASS(camel_mapi_store_parent_class)->can_refresh_folder (store, info, error) ||
-	      (camel_url_get_param (url, "check_all") != NULL);
-}
+	check_all = camel_mapi_settings_get_check_all (CAMEL_MAPI_SETTINGS (settings));
 
-static gboolean
-mapi_store_folder_is_subscribed (CamelStore *store,
-                                 const gchar *folder_name)
-{
-	CamelMapiStore *mapi_store = (CamelMapiStore *) store;
-	CamelStoreInfo *si;
-	gint truth = FALSE;
-
-	if ((si = camel_store_summary_path ((CamelStoreSummary *) mapi_store->summary, folder_name))) {
-		truth = (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) != 0;
-		camel_store_summary_info_free ((CamelStoreSummary *) mapi_store->summary, si);
-	}
-
-	return truth;
+	return CAMEL_STORE_CLASS(camel_mapi_store_parent_class)->can_refresh_folder (store, info, error) || check_all;
 }
 
 static CamelFolder *
@@ -915,7 +926,7 @@ mapi_store_get_folder_sync (CamelStore *store,
 	CamelService *service;
 	CamelStoreInfo *si;
 	CamelFolder *folder;
-	const gchar *user_data_dir;
+	const gchar *user_cache_dir;
 	gchar *storage_path;
 
 	si = camel_mapi_store_summary_full_name (mapi_store->summary, folder_name);
@@ -947,9 +958,9 @@ mapi_store_get_folder_sync (CamelStore *store,
 		camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, si);
 
 	service = CAMEL_SERVICE (store);
-	user_data_dir = camel_service_get_user_data_dir (service);
+	user_cache_dir = camel_service_get_user_cache_dir (service);
 
-	storage_path = g_strdup_printf ("%s/folders", user_data_dir);
+	storage_path = g_build_filename (user_cache_dir, "folders", NULL);
 	folder = camel_mapi_folder_new (store, folder_name, storage_path, flags, error);
 	g_free (storage_path);
 
@@ -1079,7 +1090,7 @@ mapi_store_create_folder_sync (CamelStore *store,
 		mapi_update_folder_hash_tables (mapi_store, root->full_name, fid, parent_id);
 
 		camel_store_folder_created (store, root);
-		camel_store_folder_subscribed (store, root);
+		camel_subscribable_folder_subscribed (CAMEL_SUBSCRIBABLE (store), root);
 	} else {
 		if (mapi_error) {
 			g_set_error (
@@ -1174,7 +1185,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	CamelMapiStorePrivate  *priv = mapi_store->priv;
 	CamelStoreInfo *si = NULL;
 	CamelService *service;
-	const gchar *user_data_dir;
+	const gchar *user_cache_dir;
 	gchar *old_parent, *new_parent, *tmp;
 	gboolean move_cache = TRUE;
 	const gchar *old_fid_str, *new_parent_fid_str = NULL;
@@ -1182,7 +1193,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	GError *local_error = NULL;
 
 	service = CAMEL_SERVICE (store);
-	user_data_dir = camel_service_get_user_data_dir (service);
+	user_cache_dir = camel_service_get_user_cache_dir (service);
 
 	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
@@ -1369,8 +1380,8 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	if (move_cache) {
 		gchar *oldpath, *newpath;
 
-		oldpath = g_build_filename (user_data_dir, "folders", old_name, NULL);
-		newpath = g_build_filename (user_data_dir, "folders", new_name, NULL);
+		oldpath = g_build_filename (user_cache_dir, "folders", old_name, NULL);
+		newpath = g_build_filename (user_cache_dir, "folders", new_name, NULL);
 
 		if (g_file_test (oldpath, G_FILE_TEST_IS_DIR) && g_rename (oldpath, newpath) == -1 && errno != ENOENT) {
 			g_warning ("Could not rename message cache '%s' to '%s': %s: cache reset", oldpath, newpath, g_strerror (errno));
@@ -1387,12 +1398,36 @@ mapi_store_rename_folder_sync (CamelStore *store,
 }
 
 static gboolean
-mapi_store_subscribe_folder_sync (CamelStore *store,
+mapi_store_noop_sync (CamelStore *store,
+                      GCancellable *cancellable,
+                      GError **error)
+{
+	return TRUE;
+}
+
+static gboolean
+mapi_store_folder_is_subscribed (CamelSubscribable *subscribable,
+                                 const gchar *folder_name)
+{
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (subscribable);
+	CamelStoreInfo *si;
+	gint truth = FALSE;
+
+	if ((si = camel_store_summary_path ((CamelStoreSummary *) mapi_store->summary, folder_name))) {
+		truth = (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) != 0;
+		camel_store_summary_info_free ((CamelStoreSummary *) mapi_store->summary, si);
+	}
+
+	return truth;
+}
+
+static gboolean
+mapi_store_subscribe_folder_sync (CamelSubscribable *subscribable,
                                   const gchar *folder_name,
                                   GCancellable *cancellable,
                                   GError **error)
 {
-	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (subscribable);
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si = NULL;
 	const gchar *parent_name = NULL, *use_folder_name = folder_name, *fid = NULL;
@@ -1438,7 +1473,7 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 		fi->flags |= CAMEL_FOLDER_SUBSCRIBED;
 		fi->flags |= CAMEL_FOLDER_NOCHILDREN;
 		fi->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
-		camel_store_folder_subscribed (store, fi);
+		camel_subscribable_folder_subscribed (subscribable, fi);
 		camel_folder_info_free (fi);
 	} else {
 		CamelURL *url;
@@ -1452,7 +1487,7 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 }
 
 static gboolean
-mapi_store_unsubscribe_folder_sync (CamelStore *store,
+mapi_store_unsubscribe_folder_sync (CamelSubscribable *subscribable,
                                     const gchar *folder_name,
                                     GCancellable *cancellable,
                                     GError **error)
@@ -1463,7 +1498,7 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	const gchar *fid = NULL, *use_folder_name = folder_name;
 	gchar *f_name = NULL;
 
-	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (subscribable);
 	CamelURL *url;
 
 	url = camel_service_get_camel_url (CAMEL_SERVICE (mapi_store));
@@ -1491,7 +1526,7 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	}
 	if (si->flags & CAMEL_MAPI_FOLDER_MAIL) {
 		fi = mapi_build_folder_info (mapi_store, parent_name, folder_name);
-		camel_store_folder_unsubscribed (store, fi);
+		camel_subscribable_folder_unsubscribed (subscribable, fi);
 		camel_folder_info_free (fi);
 	} else {
 		guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, use_folder_name);
@@ -1503,12 +1538,32 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	return TRUE;
 }
 
-static gboolean
-mapi_store_noop_sync (CamelStore *store,
-                      GCancellable *cancellable,
-                      GError **error)
+static void
+mapi_migrate_to_user_cache_dir (CamelService *service)
 {
-	return TRUE;
+	const gchar *user_data_dir, *user_cache_dir;
+
+	g_return_if_fail (service != NULL);
+	g_return_if_fail (CAMEL_IS_SERVICE (service));
+
+	user_data_dir = camel_service_get_user_data_dir (service);
+	user_cache_dir = camel_service_get_user_cache_dir (service);
+
+	g_return_if_fail (user_data_dir != NULL);
+	g_return_if_fail (user_cache_dir != NULL);
+
+	/* migrate only if the source directory exists and the destination doesn't */
+	if (g_file_test (user_data_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) &&
+	    !g_file_test (user_cache_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+		gchar *parent_dir;
+
+		parent_dir = g_path_get_dirname (user_cache_dir);
+		g_mkdir_with_parents (parent_dir, S_IRWXU);
+		g_free (parent_dir);
+
+		if (g_rename (user_data_dir, user_cache_dir) == -1)
+			g_debug ("%s: Failed to migrate '%s' to '%s': %s", G_STRFUNC, user_data_dir, user_cache_dir, g_strerror (errno));
+	}
 }
 
 static void
@@ -1526,16 +1581,17 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	object_class->constructed = mapi_store_constructed;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
+	service_class->settings_type = CAMEL_TYPE_MAPI_SETTINGS;
 	service_class->get_name = mapi_get_name;
 	service_class->connect_sync = mapi_connect_sync;
 	service_class->disconnect_sync = mapi_disconnect_sync;
+	service_class->authenticate_sync = mapi_authenticate_sync;
 	service_class->query_auth_types_sync = mapi_query_auth_types_sync;
 
 	store_class = CAMEL_STORE_CLASS (class);
 	store_class->hash_folder_name = mapi_store_hash_folder_name;
 	store_class->compare_folder_name = mapi_store_compare_folder_name;
 	store_class->can_refresh_folder = mapi_store_can_refresh_folder;
-	store_class->folder_is_subscribed = mapi_store_folder_is_subscribed;
 	store_class->free_folder_info = camel_store_free_folder_info_full;
 	store_class->get_folder_sync = mapi_store_get_folder_sync;
 	store_class->get_folder_info_sync = mapi_store_get_folder_info_sync;
@@ -1544,11 +1600,17 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	store_class->create_folder_sync = mapi_store_create_folder_sync;
 	store_class->delete_folder_sync = mapi_store_delete_folder_sync;
 	store_class->rename_folder_sync = mapi_store_rename_folder_sync;
-	store_class->subscribe_folder_sync = mapi_store_subscribe_folder_sync;
-	store_class->unsubscribe_folder_sync = mapi_store_unsubscribe_folder_sync;
 	store_class->noop_sync = mapi_store_noop_sync;
 }
 
+static void
+camel_subscribable_init (CamelSubscribableInterface *interface)
+{
+	interface->folder_is_subscribed = mapi_store_folder_is_subscribed;
+	interface->subscribe_folder_sync = mapi_store_subscribe_folder_sync;
+	interface->unsubscribe_folder_sync = mapi_store_unsubscribe_folder_sync;
+}
+
 /*
 ** store is already initilyse to NULL or 0 value
 ** class already have a parent_class
@@ -1568,20 +1630,19 @@ mapi_store_constructed (GObject *object)
 	CamelStore *store = CAMEL_STORE (object);
 	CamelMapiStorePrivate *priv = mapi_store->priv;
 	CamelService *service;
-	CamelURL *url;
-	const gchar *user_data_dir;
+	const gchar *user_cache_dir;
 	gchar *path = NULL;
 
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (camel_mapi_store_parent_class)->constructed (object);
 
 	service = CAMEL_SERVICE (object);
-	url = camel_service_get_camel_url (service);
-	user_data_dir = camel_service_get_user_data_dir (service);
+	mapi_migrate_to_user_cache_dir (service);
+
+	user_cache_dir = camel_service_get_user_cache_dir (service);
 
 	/*store summary*/
-	path = g_alloca (strlen (user_data_dir) + 32);
-	sprintf (path, "%s/.summary", user_data_dir);
+	path = g_build_filename (user_cache_dir, ".summary", NULL);
 
 	mapi_store->summary = camel_mapi_store_summary_new ();
 	camel_store_summary_set_filename ((CamelStoreSummary *)mapi_store->summary, path);
@@ -1589,20 +1650,6 @@ mapi_store_constructed (GObject *object)
 	camel_store_summary_touch ((CamelStoreSummary *)mapi_store->summary);
 	camel_store_summary_load ((CamelStoreSummary *) mapi_store->summary);
 
-	/*user and profile*/
-	priv->profile = g_strdup (camel_url_get_param(url, "profile"));
-
-	/*base url*/
-	priv->base_url = camel_url_to_string (
-		camel_service_get_camel_url (service),
-		CAMEL_URL_HIDE_PASSWORD |
-		CAMEL_URL_HIDE_PARAMS |
-		CAMEL_URL_HIDE_AUTH);
-
-	/*filter*/
-	if (camel_url_get_param (url, "filter"))
-		store->flags |= CAMEL_STORE_FILTER_INBOX;
-
 	/*Hash Table*/
 	priv->id_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free); /* folder ID to folder Full name */
 	priv->name_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free); /* folder Full name to folder ID */
@@ -1612,7 +1659,9 @@ mapi_store_constructed (GObject *object)
 	store->flags &= ~CAMEL_STORE_VJUNK;
 	store->flags &= ~CAMEL_STORE_VTRASH;
 
-	store->flags |= CAMEL_STORE_SUBSCRIPTIONS | CAMEL_STORE_REAL_JUNK_FOLDER;
+	store->flags |= CAMEL_STORE_REAL_JUNK_FOLDER | CAMEL_STORE_USE_CACHE_DIR;
+
+	g_free (path);
 }
 
 static char *
@@ -1633,93 +1682,6 @@ mapi_get_name(CamelService *service, gboolean brief)
 	}
 }
 
-static gboolean
-mapi_prompt_pass_creds (CamelService *service, CamelURL *url, const gchar *reason, GError **error) {
-	gchar *prompt;
-	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
-	CamelSession *session = camel_service_get_session (service);
-
-	if (reason && *reason) {
-		/* We need to un-cache the password before prompting again */
-		prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-		g_free (url->passwd);
-		url->passwd = NULL;
-	}
-
-	/*To translators : First %s : is the error text or the reason
-	  for prompting the user if it is available.
-	 Second %s is : Username.
-	 Third %s is : Server host name.*/
-	prompt = g_strdup_printf (_("%s Please enter the MAPI password for %s@%s"),
-				  reason, url->user, url->host);
-	url->passwd =
-		camel_session_get_password (session, service,
-					    prompt, "password", prompt_flags, NULL);
-	g_free (prompt);
-
-	if (!url->passwd) {
-		g_set_error (
-			error, G_IO_ERROR,
-			G_IO_ERROR_CANCELLED,
-			_("You did not enter a password."));
-		return FALSE;
-	}
-	return TRUE;
-}
-
-static gboolean
-mapi_auth_loop (CamelService *service, GError **error)
-{
-	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
-	CamelURL *url;
-	ExchangeMapiProfileData empd = { 0 };
-
-	gchar *errbuf = NULL;
-	gboolean authenticated = FALSE, ret, krb_requested = FALSE;
-
-	url = camel_service_get_camel_url (service);
-	exchange_mapi_util_profiledata_from_camelurl (&empd, url);
-
-	url->passwd = NULL;
-
-	while (!authenticated) {
-		GError *mapi_error = NULL;
-
-		if (!url->passwd) {
-			const gchar *why = errbuf ? errbuf : "";
-
-			if (empd.krb_sso) {
-				if (!krb_requested) {
-					ret = exchange_mapi_util_trigger_krb_auth (&empd, error);
-					krb_requested = TRUE;
-				}
-			} else {
-				ret = mapi_prompt_pass_creds (service, url,
-							      why, error);
-			}
-
-			g_free (errbuf);
-			errbuf = NULL;
-		}
-		if (!ret || (error && *error)) {
-			return FALSE;
-		}
-
-		store->priv->conn = exchange_mapi_connection_new (store->priv->profile, url->passwd, &mapi_error);
-		if (!store->priv->conn || !exchange_mapi_connection_connected (store->priv->conn)) {
-			if (mapi_error) {
-				errbuf = g_strdup_printf (_("Unable to authenticate to Exchange MAPI server: %s"), mapi_error->message);
-				g_error_free (mapi_error);
-			} else {
-				errbuf = g_strdup (_("Unable to authenticate to Exchange MAPI server"));
-			}
-		} else
-			authenticated = TRUE;
-
-	}
-	return TRUE;
-}
-
 static gboolean
 mapi_connect_sync (CamelService *service,
                    GCancellable *cancellable,
@@ -1727,8 +1689,11 @@ mapi_connect_sync (CamelService *service,
 {
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
 	CamelServiceConnectionStatus status;
+	CamelSession *session;
 	guint16 event_mask = 0;
 
+	session = camel_service_get_session (service);
+
 	status = camel_service_get_connection_status (service);
 	if (status == CAMEL_SERVICE_DISCONNECTED) {
 		return FALSE;
@@ -1740,7 +1705,7 @@ mapi_connect_sync (CamelService *service,
 		return TRUE;
 	}
 
-	if (!mapi_auth_loop (service, error)) {
+	if (!camel_session_authenticate_sync (session, service, NULL, cancellable, error)) {
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		camel_service_disconnect_sync (service, TRUE, NULL);
 		return FALSE;
@@ -1798,6 +1763,67 @@ mapi_disconnect_sync (CamelService *service,
 	return TRUE;
 }
 
+static CamelAuthenticationResult
+mapi_authenticate_sync (CamelService *service,
+                        const gchar *mechanism,
+                        GCancellable *cancellable,
+                        GError **error)
+{
+	CamelAuthenticationResult result;
+	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
+	CamelURL *url;
+	CamelSettings *settings;
+	CamelMapiSettings *mapi_settings;
+	ExchangeMapiProfileData empd = { 0 };
+	const gchar *profile;
+	const gchar *password;
+	GError *mapi_error = NULL;
+
+	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
+	mapi_settings = CAMEL_MAPI_SETTINGS (settings);
+
+	empd.server = url->host;
+	empd.username = url->user;
+	exchange_mapi_util_profiledata_from_settings (&empd, mapi_settings);
+
+	profile = camel_mapi_settings_get_profile (mapi_settings);
+
+	if (empd.krb_sso) {
+		if (exchange_mapi_util_trigger_krb_auth (&empd, error))
+			return CAMEL_AUTHENTICATION_ACCEPTED;
+		else
+			return CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	password = camel_service_get_password (service);
+
+	if (password == NULL) {
+		g_set_error_literal (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			_("Authentication password not available"));
+		return CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	store->priv->conn = exchange_mapi_connection_new (profile, password, &mapi_error);
+	if (store->priv->conn && exchange_mapi_connection_connected (store->priv->conn)) {
+		result = CAMEL_AUTHENTICATION_ACCEPTED;
+	} else if (g_error_matches (mapi_error, E_MAPI_ERROR, MAPI_E_LOGON_FAILED)) {
+		g_clear_error (&mapi_error);
+		result = CAMEL_AUTHENTICATION_REJECTED;
+	} else {
+		/* mapi_error should be set */
+		g_return_val_if_fail (
+			mapi_error != NULL,
+			CAMEL_AUTHENTICATION_ERROR);
+		g_propagate_error (error, mapi_error);
+		result = CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	return result;
+}
+
 static GList *
 mapi_query_auth_types_sync (CamelService *service,
                             GCancellable *cancellable,
@@ -2079,18 +2105,6 @@ camel_mapi_store_folder_lookup (CamelMapiStore *mapi_store, const gchar *folder_
 	return g_hash_table_lookup (priv->id_hash, folder_id);
 }
 
-const gchar *
-camel_mapi_store_get_profile_name (CamelMapiStore *mapi_store)
-{
-	CamelMapiStorePrivate *priv;
-
-	g_return_val_if_fail (CAMEL_IS_MAPI_STORE (mapi_store), NULL);
-
-	priv = mapi_store->priv;
-
-	return priv->profile;
-}
-
 ExchangeMapiConnection *
 camel_mapi_store_get_exchange_connection (CamelMapiStore *mapi_store)
 {
